@@ -21,12 +21,22 @@
 #' threshold to count an excursion.
 #' @param magedef How large an excursion needs to be in order to count in the 
 #' MAGE calculation (e.g. greater than 1 standard deviation).
+#' @param congan CONGA interval in hours.
+#' @param daystart The numeric hour at which daytime should start (e.g. to start 
+#' counting day time at 6:00am, set daystart = 6).
+#' @param dayend The numeric hour at which daytime should end (this parameter 
+#' uses military time, so to stop counting day time at 10:00pm, set dayend = 22).
+#' @param format Whether observations are in rows or columns.
 #' @usage cgmvariables(inputdirectory,
 #' outputdirectory = tempdir(),
 #' outputname = "REDCap Upload",
 #' aboveexcursionlength = 35,
 #' belowexcursionlength = 10,
-#' magedef = "1sd")
+#' magedef = "1sd",
+#' congan = 1,
+#' daystart = 6,
+#' dayend = 22,
+#' format = "rows")
 #' @examples cgmvariables(system.file("extdata","Cleaned",package = "cgmanalysis"))
 #' @return A data frame containing calculated CGM variables, with each column
 #' representing one CGM file.
@@ -37,24 +47,30 @@ cgmvariables <- function(inputdirectory,
                          outputname = "REDCap Upload",
                          aboveexcursionlength = 35,
                          belowexcursionlength = 10,
-                         magedef = "1sd") {
+                         magedef = "1sd",
+                         congan = 1,
+                         daystart = 6,
+                         dayend = 22,
+                         format = "rows") {
 
 # Read in data, create results dataframe. The dataframe has one column for each 
 # file in the input directory, and is desgined to be uploaded to REDCap. 
   files <- base::list.files(path = inputdirectory,full.names = TRUE)
-  cgmupload <- base::data.frame(base::matrix(nrow = 0,ncol = base::length(files)))
+  cgmupload <- 
+    base::as.data.frame(base::matrix(nrow = 0,ncol = base::length(files)))
   base::colnames(cgmupload) <- base::rep("Record",base::length(files))
 # Define the order in which lubridate parses dates.  
   dateparseorder <- c("mdy HM","mdy HMS","mdY HM","mdY HMS","dmy HM","dmy HMS",
                       "dmY HM","dmY HMS","Ymd HM","Ymd HMS","ymd HM","ymd HMS",
                       "Ydm HM","Ydm HMS","ydm HM","ydm HMS")
+  allhours <- 0:23
 # Iterate through the input directory and calculate CGM variables for each file.
 # The cgmvariables() function only works on CSV files that have been cleaned by 
 # cleandata(), or that have been manually edited and fit the format of
 # cleandata() output. 
   for (f in 1:base::length(files)) {    
 # Basic variables
-    table <- utils::read.csv(files[f],stringsAsFactors = FALSE)
+    table <- utils::read.csv(files[f],stringsAsFactors = FALSE,na.strings = "")
     
     cgmupload["subject_id",f] <- table$subjectid[1]
 # Format columns.    
@@ -63,18 +79,16 @@ cgmvariables <- function(inputdirectory,
                                                   dateparseorder,tz = "UTC"))
     table$sensorglucose <- base::as.numeric(table$sensorglucose)
     interval <- pracma::Mode(base::diff(base::as.numeric(table$timestamp)))
+    interval <- base::abs(interval)
     cgmupload["date_cgm_placement", f] <- 
-      as.character(as.Date(table$subjectid[2],format = "%m/%d/%Y %T"))
-    table$subjectid[2] <- 
-      base::as.POSIXct(lubridate::parse_date_time(table$subjectid[2],
-                                                  dateparseorder),tz = "UTC")
-    table$subjectid[3] <- 
-      base::as.POSIXct(lubridate::parse_date_time(table$subjectid[3],
-                                                  dateparseorder),tz = "UTC")
+      base::as.character(min(table$timestamp,na.rm = T))
     
-    totaltime <- as.numeric(table$subjectid[3])-as.numeric(table$subjectid[2])
+    totaltime <- 
+      base::as.numeric(base::difftime(base::max(table$timestamp, na.rm = T),
+                                      base::min(table$timestamp,na.rm = T),
+                                      units = "secs"))
     cgmupload["percent_cgm_wear",f] <- 
-      round(((length(table$sensorglucose)/(totaltime/interval))*100),2)
+      base::round(((base::length(table$sensorglucose)/(totaltime/interval))*100),2)
     
     table <- table[,-c(1)]
     table <- table[stats::complete.cases(table),]
@@ -84,10 +98,12 @@ cgmvariables <- function(inputdirectory,
     cgmupload["total_sensor_readings",f] <- 
       base::as.numeric(base::length(base::which(!is.na(table$sensorglucose))))
     cgmupload["average_sensor",f] <- 
-      base::mean(table$sensorglucose[base::which(!is.na(table$sensorglucose))])
+      base::mean(table$sensorglucose[base::which(!is.na(table$sensorglucose))],na.rm = T)
     cgmupload["estimated_a1c",f] <- 
       base::round((46.7 + (base::mean(table$sensorglucose[
         base::which(!is.na(table$sensorglucose))]))) / 28.7,digits = 1)
+    cgmupload["gmi",f] <- base::round(3.31 + (0.02392 * base::mean(table$sensorglucose[
+      base::which(!is.na(table$sensorglucose))])), digits = 1)
     cgmupload["q1_sensor",f] <- 
       base::as.numeric(base::summary(table$sensorglucose[
         base::which(!is.na(table$sensorglucose))])[2])
@@ -109,8 +125,8 @@ cgmvariables <- function(inputdirectory,
     
 # Excursions over 120 calculations.
     BGover120 <- 
-      base::as.numeric(table$sensorglucose[base::which(!is.na(table$sensorglucose))],
-                       length = 1)
+      base::as.numeric(table$sensorglucose[base::which(!is.na(
+        table$sensorglucose))],length = 1)
     BGover120[BGover120 < 120] <- 0
     BGover120[BGover120 >= 120] <- 1
     BG120.rle <- base::rle(BGover120)
@@ -118,7 +134,8 @@ cgmvariables <- function(inputdirectory,
       base::as.numeric(BG120.rle$lengths[base::which(BG120.rle$values == 1)])
     
     cgmupload["excursions_over_120",f] <- 
-      base::length(base::which(excursions120 > ((aboveexcursionlength * 60)/interval)))
+      base::length(base::which(excursions120 > 
+                                 ((aboveexcursionlength * 60)/interval)))
     cgmupload["min_spent_over_120",f] <- base::sum(BGover120) * (interval/60)
     cgmupload["percent_time_over_120",f] <- 
       ((base::sum(BGover120) * (interval/60))/
@@ -126,8 +143,8 @@ cgmvariables <- function(inputdirectory,
     
 # Over 140.
     BGover140 <- 
-      base::as.numeric(table$sensorglucose[base::which(!is.na(table$sensorglucose))],
-                 length = 1)
+      base::as.numeric(table$sensorglucose[base::which(!is.na(
+        table$sensorglucose))],length = 1)
     BGover140[BGover140 < 140] <- 0
     BGover140[BGover140 >= 140] <- 1
     BG140.rle <- base::rle(BGover140)
@@ -135,7 +152,8 @@ cgmvariables <- function(inputdirectory,
       base::as.numeric(BG140.rle$lengths[base::which(BG140.rle$values == 1)])
     
     cgmupload["excursions_over_140",f] <- 
-      base::length(base::which(excursions140 > ((aboveexcursionlength * 60)/interval)))
+      base::length(base::which(excursions140 > 
+                                 ((aboveexcursionlength * 60)/interval)))
     cgmupload["min_spent_over_140",f] <- base::sum(BGover140) * (interval/60)
     cgmupload["percent_time_over_140",f] <- 
       ((base::sum(BGover140) * (interval/60))/
@@ -143,8 +161,8 @@ cgmvariables <- function(inputdirectory,
     
 # Over 180.
     BGover180 <- 
-      base::as.numeric(table$sensorglucose[base::which(!is.na(table$sensorglucose))],
-                 length = 1)
+      base::as.numeric(table$sensorglucose[base::which(!is.na(
+        table$sensorglucose))],length = 1)
     BGover180[BGover180 < 180] <- 0
     BGover180[BGover180 >= 180] <- 1
     BG180.rle <- base::rle(BGover180)
@@ -152,7 +170,8 @@ cgmvariables <- function(inputdirectory,
       base::as.numeric(BG180.rle$lengths[base::which(BG180.rle$values == 1)])
     
     cgmupload["excursions_over_180",f] <- 
-      base::length(base::which(excursions180 > ((aboveexcursionlength * 60)/interval)))
+      base::length(base::which(excursions180 > 
+                                 ((aboveexcursionlength * 60)/interval)))
     cgmupload["min_spent_over_180",f] <- base::sum(BGover180) * (interval/60)
     cgmupload["percent_time_over_180",f] <- 
       ((base::sum(BGover180) * (interval/60))/
@@ -160,8 +179,8 @@ cgmvariables <- function(inputdirectory,
     
 # Over 200.
     BGover200 <- 
-      base::as.numeric(table$sensorglucose[base::which(!is.na(table$sensorglucose))],
-                 length = 1)
+      base::as.numeric(table$sensorglucose[base::which(!is.na(
+        table$sensorglucose))],length = 1)
     BGover200[BGover200 < 200] <- 0
     BGover200[BGover200 >= 200] <- 1
     BG200.rle <- base::rle(BGover200)
@@ -169,7 +188,8 @@ cgmvariables <- function(inputdirectory,
       base::as.numeric(BG200.rle$lengths[base::which(BG200.rle$values == 1)])
     
     cgmupload["excursions_over_200",f] <- 
-      base::length(base::which(excursions200 > ((aboveexcursionlength * 60)/interval)))
+      base::length(base::which(excursions200 > 
+                                 ((aboveexcursionlength * 60)/interval)))
     cgmupload["min_spent_over_200",f] <- base::sum(BGover200) * (interval/60)
     cgmupload["percent_time_over_200",f] <- 
       ((base::sum(BGover200) * (interval/60))/
@@ -184,8 +204,8 @@ cgmvariables <- function(inputdirectory,
         
 # Over 250.
     BGover250 <- 
-      base::as.numeric(table$sensorglucose[base::which(!is.na(table$sensorglucose))],
-                 length = 1)
+      base::as.numeric(table$sensorglucose[base::which(!is.na(
+        table$sensorglucose))],length = 1)
     BGover250[BGover250 < 250] <- 0
     BGover250[BGover250 >= 250] <- 1
     BG250.rle <- base::rle(BGover250)
@@ -193,7 +213,8 @@ cgmvariables <- function(inputdirectory,
       base::as.numeric(BG250.rle$lengths[base::which(BG250.rle$values == 1)])
     
     cgmupload["excursions_over_250",f] <- 
-      base::length(base::which(excursions250 > ((aboveexcursionlength * 60)/interval)))
+      base::length(base::which(excursions250 > 
+                                 ((aboveexcursionlength * 60)/interval)))
     cgmupload["min_spent_over_250",f] <- base::sum(BGover250) * (interval/60)
     cgmupload["percent_time_over_250",f] <- 
       ((base::sum(BGover250) * (interval/60))/
@@ -201,8 +222,8 @@ cgmvariables <- function(inputdirectory,
     
 # Under 54.
     BGunder54 <- 
-      base::as.numeric(table$sensorglucose[base::which(!is.na(table$sensorglucose))],
-                 length = 1)
+      base::as.numeric(table$sensorglucose[base::which(!is.na(
+        table$sensorglucose))],length = 1)
     BGunder54[BGunder54 <= 54] <- 1
     BGunder54[BGunder54 > 54] <- 0
     BG54.rle <- base::rle(BGunder54)
@@ -210,7 +231,8 @@ cgmvariables <- function(inputdirectory,
       base::as.numeric(BG54.rle$lengths[base::which(BG54.rle$values == 1)])
     
     cgmupload["excursions_under_54",f] <- 
-      base::length(base::which(excursions54 > ((belowexcursionlength * 60)/interval)))
+      base::length(base::which(excursions54 > 
+                                 ((belowexcursionlength * 60)/interval)))
     cgmupload["min_spent_under_54",f] <- base::sum(BGunder54) * (interval/60)
     cgmupload["percent_time_under_54",f] <- 
       ((base::sum(BGunder54) * (interval/60))/
@@ -218,8 +240,8 @@ cgmvariables <- function(inputdirectory,
     
 # Under 60.
     BGunder60 <- 
-      base::as.numeric(table$sensorglucose[base::which(!is.na(table$sensorglucose))],
-                 length = 1)
+      base::as.numeric(table$sensorglucose[base::which(!is.na(
+        table$sensorglucose))],length = 1)
     BGunder60[BGunder60 <= 60] <- 1
     BGunder60[BGunder60 > 60] <- 0
     BG60.rle <- base::rle(BGunder60)
@@ -227,7 +249,8 @@ cgmvariables <- function(inputdirectory,
       base::as.numeric(BG60.rle$lengths[base::which(BG60.rle$values == 1)])
     
     cgmupload["excursions_under_60",f] <- 
-      base::length(base::which(excursions60 > ((belowexcursionlength * 60)/interval)))
+      base::length(base::which(excursions60 > 
+                                 ((belowexcursionlength * 60)/interval)))
     cgmupload["min_spent_under_60",f] <- base::sum(BGunder60) * (interval/60)
     cgmupload["percent_time_under_60",f] <- 
       ((base::sum(BGunder60) * (interval/60))/
@@ -244,18 +267,31 @@ cgmvariables <- function(inputdirectory,
       base::as.numeric(BG70.rle$lengths[base::which(BG70.rle$values == 1)])
     
     cgmupload["excursions_under_70",f] <- 
-      base::length(base::which(excursions70 > ((belowexcursionlength * 60)/interval)))
+      base::length(base::which(excursions70 > 
+                                 ((belowexcursionlength * 60)/interval)))
     cgmupload["min_spent_under_70",f] <- base::sum(BGunder70) * (interval/60)
     cgmupload["percent_time_under_70",f] <- 
       ((base::sum(BGunder70) * (interval/60))/
          (base::length(table$sensorglucose) * (interval/60))) * 100
     
+# Time in range.
+    BGinrange <- 
+      base::as.numeric(table$sensorglucose[base::which(!is.na(table$sensorglucose))],
+                       length = 1)
+    BGinrange <- ifelse(BGinrange %in% 70:180, 1,0)
+    cgmupload["min_spent_70_180",f] <- base::sum(BGinrange) * (interval/60)
+    cgmupload["percent_time_70_180",f] <- 
+      ((base::sum(BGinrange) * (interval/60))/
+         (base::length(table$sensorglucose) * (interval/60))) * 100
+    
 # Find daytime AUC.
     daytime_indexes <- 
-      base::which(base::as.numeric(base::format(table$timestamp,"%H")) %in% 6:22)
+      base::which(base::as.numeric(base::format(table$timestamp,"%H")) %in% 
+                    daystart:dayend)
     daytime_sensor <- table$sensorglucose[daytime_indexes]
     xaxis <- 
-      base::seq(from = 0, length.out = base::length(daytime_sensor),by = (interval / 60))
+      base::seq(from = 0, length.out = base::length(daytime_sensor),by = 
+                  (interval / 60))
     
 # Remove NAs if they are present.
     xaxis[base::which(is.na(daytime_sensor))] <- NA
@@ -265,21 +301,24 @@ cgmvariables <- function(inputdirectory,
     cgmupload["daytime_auc",f] <- aucs[base::length(daytime_sensor)]
     
 # Other daytime sensor glucose variables.
-    cgmupload["daytime_avg_sensor_glucose",f] <- base::mean(stats::na.omit(daytime_sensor))
-    cgmupload["daytime_min_sensor_glucose",f] <- min(daytime_sensor)
-    cgmupload["daytime_max_sensor_glucose",f] <- max(daytime_sensor)
+    cgmupload["daytime_avg_sensor_glucose",f] <- 
+      base::mean(stats::na.omit(daytime_sensor))
+    cgmupload["daytime_min_sensor_glucose",f] <- base::min(daytime_sensor)
+    cgmupload["daytime_max_sensor_glucose",f] <- base::max(daytime_sensor)
     cgmupload["daytime_sd",f] <- stats::sd(daytime_sensor)
     
 # Nighttime AUC.
     nighttime_indexes <- 
-      base::which(base::as.numeric(base::format(table$timestamp,"%H")) %in% c(23,24,0:5))
+      base::which(base::as.numeric(base::format(table$timestamp,"%H")) %in% 
+                    allhours[base::which(!(0:23 %in% daystart:dayend))])
     nighttime_sensor <- table$sensorglucose[nighttime_indexes]
     xaxis <- 
-      base::seq(from = 0, length.out = base::length(nighttime_indexes),by = (interval / 60))
+      base::seq(from = 0, length.out = base::length(nighttime_indexes),by = 
+                  (interval / 60))
     
 # Day/night ratio.
     cgmupload["day_night_sensor_ratio",f] <- 
-      round(length(daytime_sensor)/length(nighttime_sensor),1)
+      base::round(base::length(daytime_sensor)/base::length(nighttime_sensor),1)
     
 # Remove NAs if they are present.
     xaxis[base::which(is.na(nighttime_sensor))] <- NA
@@ -291,14 +330,15 @@ cgmvariables <- function(inputdirectory,
 # Other nighttime sensor glucose variables.
     cgmupload["nighttime_avg_sens_glucose",f] <- 
       base::mean(stats::na.omit(nighttime_sensor))
-    cgmupload["nighttime_min_sens_glucose",f] <- min(nighttime_sensor)
-    cgmupload["nighttime_max_sens_glucose",f] <- max(nighttime_sensor)
+    cgmupload["nighttime_min_sens_glucose",f] <- base::min(nighttime_sensor)
+    cgmupload["nighttime_max_sens_glucose",f] <- base::max(nighttime_sensor)
     cgmupload["nighttime_sd",f] <- stats::sd(nighttime_sensor)
     
 # Total AUC.
     sensorBG <- base::as.numeric(table$sensorglucose,length = 1)
     xaxis <- 
-      base::seq(from = 0, length.out = base::length(sensorBG),by = (interval / 60))
+      base::seq(from = 0, length.out = base::length(sensorBG),by = 
+                  (interval / 60))
     
 # Remove NAs if they are present.
     xaxis[base::which(is.na(sensorBG))] <- NA
@@ -316,7 +356,8 @@ cgmvariables <- function(inputdirectory,
     sensorover180 <- sensorover180[sensorover180 >= 180]
     sensorover180 <- sensorover180[!is.na(sensorover180)]
     xaxis <- 
-      base::seq(from = 0, length.out = base::length(sensorover180),by = (interval / 60))
+      base::seq(from = 0, length.out = base::length(sensorover180),by = 
+                  (interval / 60))
     
 # Calculate cumulative AUC, and subtract recatangle where length = 180 &
 # width = minutes.
@@ -336,10 +377,14 @@ cgmvariables <- function(inputdirectory,
 # Smooth data using an exponentially weighted moving average, calculate SD of 
 # unsmoothed data.  
     table$smoothed <- 
-      base::as.numeric(zoo::rollapply(zoo::zoo(table$sensorglucose), 9, function(x) c(1,2,4,8,16,8,4,2,1) %*% (x / 46),fill = NA))
+      base::as.numeric(zoo::rollapply(zoo::zoo(table$sensorglucose), 9, 
+                                      function(x) c(1,2,4,8,16,8,4,2,1) %*% 
+                                        (x / 46),fill = NA))
     table$smoothed[1:4] <- base::mean(stats::na.omit(table$sensorglucose[1:4]))
-    table$smoothed[(base::length(table$smoothed)-3):base::length(table$smoothed)] <- 
-      base::mean(table$sensorglucose[(base::length(table$sensorglucose)-3):base::length(table$sensorglucose)])
+    table$smoothed[(base::length(table$smoothed)-3):
+                     base::length(table$smoothed)] <- 
+      base::mean(table$sensorglucose[(base::length(table$sensorglucose)-3):
+                                       base::length(table$sensorglucose)])
     
     sd <- stats::sd(table$sensorglucose)
 # Identify turning points, peaks, and nadirs.
@@ -349,7 +394,8 @@ cgmvariables <- function(inputdirectory,
 # Calculate the difference between each nadir and its following peak. If the     
 # data starts on a peak, remove it. Otherwise remove the final pit to create an 
 # even number of pits and peaks.
-    if (tpoints[["firstispeak"]] == TRUE && base::length(peaks) != base::length(pits)) {
+    if (tpoints[["firstispeak"]] == TRUE && base::length(peaks) != 
+        base::length(pits)) {
       peaks <- peaks[2:base::length(peaks)]
     } else if (tpoints[["firstispeak"]] == FALSE && 
                base::length(peaks) != base::length(pits)) {
@@ -357,26 +403,45 @@ cgmvariables <- function(inputdirectory,
     }
     differences <- table$sensorglucose[peaks] - table$sensorglucose[pits]
     
-# Calculate the average of the differences greater than the entire dataset SD, 2SD, etc.
+# Calculate the average of the differences greater than the entire dataset 
+# SD, 2SD, etc.
     if (magedef == "1sd") {
       cgmupload["r_mage",f] <- 
         base::mean(stats::na.omit(differences[base::which(differences > sd)]))
     } else if (magedef == "1.5sd") {
       cgmupload["r_mage",f] <- 
-        base::mean(stats::na.omit(differences[base::which(differences > (sd * 1.5))]))
+        base::mean(stats::na.omit(differences[base::which(differences > 
+                                                            (sd * 1.5))]))
     } else if ( magedef == "2sd") {
       cgmupload["r_mage",f] <- 
-        base::mean(stats::na.omit(differences[base::which(differences > (sd * 2))]))
+        base::mean(stats::na.omit(differences[base::which(differences > 
+                                                            (sd * 2))]))
     } else {
       cgmupload["r_mage",f] <- 
-        base::mean(stats::na.omit(differences[base::which(differences > magedef)]))
+        base::mean(stats::na.omit(differences[base::which(differences > 
+                                                            magedef)]))
     }
     
+#J-index
+    cgmupload["j_index",f] <- 
+      0.324 * (base::mean(table$sensorglucose, na.rm = T) + 
+                 stats::sd(table$sensorglucose, na.rm = T))^2
+# CONGA    
+    n <- (congan * 3600)
+    conga.times <- table$timestamp + n
+    conga.times <- conga.times[!is.na(conga.times)]
+    conga.times <- conga.times[base::order(conga.times)]
+    conga.times <- conga.times[base::which(conga.times %in% table$timestamp)]
+    begin.times <- conga.times - n
+    congas <- table$sensorglucose[base::which(table$timestamp %in% conga.times)] - 
+      table$sensorglucose[base::which(table$timestamp %in% begin.times)]
+    cgmupload[base::paste0("conga_",congan),f] <- stats::sd(congas,na.rm = T)
 # MODD.
     table$time <- lubridate::round_date(table$timestamp,"5 minutes")
     table$time <- base::strftime(table$time, format = "%H:%M",tz = "UTC")
     moddtable <- 
-      base::data.frame(base::matrix(ncol = 2,nrow = base::length(unique(table$time))))
+      base::data.frame(base::matrix(ncol = 2,nrow = 
+                                      base::length(unique(table$time))))
     base::colnames(moddtable) <- c("time","mean_differences")
     moddtable$time <- base::unique(table$time)
 # For each time, calculate differences (absolute values) and average them.   
@@ -386,7 +451,8 @@ cgmvariables <- function(inputdirectory,
           base::which(table$time == moddtable$time[r])])))
     }
 # Average the averages.
-    cgmupload["modd",f] <- base::mean(stats::na.omit(moddtable$mean_differences))
+    cgmupload["modd",f] <- 
+      base::mean(stats::na.omit(moddtable$mean_differences))
     
 # LBGI and HBGI (based on dc1386 appendix)
     a <- 1.084
@@ -403,6 +469,10 @@ cgmvariables <- function(inputdirectory,
 # Write file.
   cgmupload <- 
     base::cbind("Variable / Field Name" = rownames(cgmupload),cgmupload)
+  if (format == "rows") {
+    cgmupload <- base::as.data.frame(base::t(cgmupload))
+    cgmupload <- cgmupload[-1,]
+  }
   filename <- base::paste(outputdirectory,"/",outputname,".csv",sep = "")
   utils::write.csv(cgmupload, file = filename,row.names = FALSE)
 }
